import numpy as np
import matplotlib.pyplot as plt

"""
Given a list of coordinates, convert them to screen space.
"""

def compute_pixel_coordinates(
    world_point: np.ndarray,
    transformation_matrix: np.ndarray,
    canvas_width: float,
    canvas_height: float,
    image_width: int,
    image_height: int
) -> np.ndarray:
    
    # Convert to camera space
    camera_point =  np.append(world_point, 1) @ transformation_matrix

    screen_x = camera_point[0] / -camera_point[2]
    screen_y = camera_point[1] / -camera_point[2]

    normalized_x = (screen_x + (canvas_width / 2)) / canvas_width
    normalized_y = (screen_y + (canvas_height / 2)) / canvas_height

    raster_x = np.floor(normalized_x * image_width)
    raster_y = np.floor(normalized_y * image_height)


    return np.array([raster_x, raster_y])

CANVAS_WIDTH = 2.0
CANVAS_HEIGHT = 2.0
IMAGE_WIDTH = 640
IMAGE_HEIGHT = 480

def main():
    points = [
        np.array([   0,  0.5, -1]),
        np.array([ 0.5,    0, -1]),
        np.array([-0.5,    0, -1])
    ]

    # Identity matrix
    camera_to_world = np.array(
       [[1, 0, 0, 0], 
        [0, 1, 0, 0], 
        [0, 0, 1, 0], 
        [0, 0, 0, 1]]
    )

    world_to_camera = np.linalg.inv(camera_to_world)

    print(points)
    print(world_to_camera)

    transformed_points = np.array([compute_pixel_coordinates(p, world_to_camera, CANVAS_WIDTH, CANVAS_HEIGHT, IMAGE_WIDTH, IMAGE_HEIGHT) for p in points])

    print(transformed_points)

    x, y = transformed_points.T
    plt.scatter(x, y)
    plt.ylim((0, IMAGE_HEIGHT))
    plt.xlim((0, IMAGE_WIDTH))
    plt.savefig("fig.png")

if __name__ == "__main__":
    main()