import fp32_pkg::*;

module fp32_mul(
    input clk,

    input i_valid,
    
    input fp32_t a,
    input fp32_t b,
    output fp32_t z,

    output logic o_valid
);

fp32_t result, result_next;

logic valid_temp;

logic sign_temp, sign_temp_next;
logic [fp32_exp_width:0] exp_temp, exp_temp_next;
logic [(fp32_mantissa_width+1)*2-1:0] mantissa_temp, mantissa_temp_next;

logic is_zero, is_zero_next;

logic round_up;

assign z = result;

always_ff @(posedge clk) begin
    valid_temp <= i_valid;
    o_valid <= valid_temp;

    is_zero <= is_zero_next;

    result <= result_next;
    exp_temp <= exp_temp_next;
    sign_temp <= sign_temp_next;
    mantissa_temp <= mantissa_temp_next;
end

always_comb begin
    sign_temp_next = a.sign ^ b.sign;

    exp_temp_next = a.exponent + b.exponent - fp32_exp_bias;
    mantissa_temp_next = ({1'b1, a.mantissa} * {1'b1, b.mantissa});

    if ((a.exponent == '0 && a.mantissa == '0) || b.exponent == '0 && b.mantissa == '0) begin
        is_zero_next = '1;
    end else begin
        is_zero_next = '0;
    end

    result_next.sign = sign_temp;

    casez (mantissa_temp[47:46])

    2'b00: begin 
        round_up = mantissa_temp[21] & |mantissa_temp[20:0];
        result_next.exponent = exp_temp[fp32_exp_width-1:0] - 1;
        result_next.mantissa = mantissa_temp[fp32_mantissa_width - 1 +: fp32_mantissa_width] + 23'(round_up);
    end

    2'b01: begin
        round_up = mantissa_temp[22] & |mantissa_temp[21:0];
        result_next.exponent = exp_temp[fp32_exp_width-1:0];
        result_next.mantissa = mantissa_temp[fp32_mantissa_width +: fp32_mantissa_width] + 23'(round_up);
    end

    2'b1?: begin
        round_up = mantissa_temp[23] & |mantissa_temp[22:0];
        result_next.exponent = exp_temp[fp32_exp_width-1:0] + 1;
        result_next.mantissa = mantissa_temp[fp32_mantissa_width + 1 +: fp32_mantissa_width] + 23'(round_up);
    end

    endcase

    if (is_zero) begin
        result_next.exponent = '0;
        result_next.mantissa = '0;
    end
end

endmodule