/*

Copyright (c) 2021 Alex Forencich

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

// Language: Verilog 2001

`resetall
`timescale 1ns / 1ps
`default_nettype none

/*
 * AXI4 lite 5x2 crossbar (wrapper)
 */
module axil_crossbar_wrap_5x2 #
(
    // Width of data bus in bits
    parameter DATA_WIDTH = 32,
    // Width of address bus in bits
    parameter ADDR_WIDTH = 32,
    // Width of wstrb (width of data bus in words)
    parameter STRB_WIDTH = (DATA_WIDTH/8),
    // Number of concurrent operations
    parameter S00_ACCEPT = 16,
    // Number of concurrent operations
    parameter S01_ACCEPT = 16,
    // Number of concurrent operations
    parameter S02_ACCEPT = 16,
    // Number of concurrent operations
    parameter S03_ACCEPT = 16,
    // Number of concurrent operations
    parameter S04_ACCEPT = 16,
    // Number of regions per master interface
    parameter M_REGIONS = 1,
    // Master interface base addresses
    // M_REGIONS concatenated fields of ADDR_WIDTH bits
    parameter M00_BASE_ADDR = 0,
    // Master interface address widths
    // M_REGIONS concatenated fields of 32 bits
    parameter M00_ADDR_WIDTH = {M_REGIONS{32'd24}},
    // Read connections between interfaces
    // S_COUNT bits
    parameter M00_CONNECT_READ = 5'b11111,
    // Write connections between interfaces
    // S_COUNT bits
    parameter M00_CONNECT_WRITE = 5'b11111,
    // Number of concurrent operations for each master interface
    parameter M00_ISSUE = 16,
    // Secure master (fail operations based on awprot/arprot)
    parameter M00_SECURE = 0,
    // Master interface base addresses
    // M_REGIONS concatenated fields of ADDR_WIDTH bits
    parameter M01_BASE_ADDR = 0,
    // Master interface address widths
    // M_REGIONS concatenated fields of 32 bits
    parameter M01_ADDR_WIDTH = {M_REGIONS{32'd24}},
    // Read connections between interfaces
    // S_COUNT bits
    parameter M01_CONNECT_READ = 5'b11111,
    // Write connections between interfaces
    // S_COUNT bits
    parameter M01_CONNECT_WRITE = 5'b11111,
    // Number of concurrent operations for each master interface
    parameter M01_ISSUE = 16,
    // Secure master (fail operations based on awprot/arprot)
    parameter M01_SECURE = 0,
    // Slave interface AW channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S00_AW_REG_TYPE = 0,
    // Slave interface W channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S00_W_REG_TYPE = 0,
    // Slave interface B channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S00_B_REG_TYPE = 1,
    // Slave interface AR channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S00_AR_REG_TYPE = 0,
    // Slave interface R channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S00_R_REG_TYPE = 2,
    // Slave interface AW channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S01_AW_REG_TYPE = 0,
    // Slave interface W channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S01_W_REG_TYPE = 0,
    // Slave interface B channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S01_B_REG_TYPE = 1,
    // Slave interface AR channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S01_AR_REG_TYPE = 0,
    // Slave interface R channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S01_R_REG_TYPE = 2,
    // Slave interface AW channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S02_AW_REG_TYPE = 0,
    // Slave interface W channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S02_W_REG_TYPE = 0,
    // Slave interface B channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S02_B_REG_TYPE = 1,
    // Slave interface AR channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S02_AR_REG_TYPE = 0,
    // Slave interface R channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S02_R_REG_TYPE = 2,
    // Slave interface AW channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S03_AW_REG_TYPE = 0,
    // Slave interface W channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S03_W_REG_TYPE = 0,
    // Slave interface B channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S03_B_REG_TYPE = 1,
    // Slave interface AR channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S03_AR_REG_TYPE = 0,
    // Slave interface R channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S03_R_REG_TYPE = 2,
    // Slave interface AW channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S04_AW_REG_TYPE = 0,
    // Slave interface W channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S04_W_REG_TYPE = 0,
    // Slave interface B channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S04_B_REG_TYPE = 1,
    // Slave interface AR channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S04_AR_REG_TYPE = 0,
    // Slave interface R channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter S04_R_REG_TYPE = 2,
    // Master interface AW channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M00_AW_REG_TYPE = 1,
    // Master interface W channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M00_W_REG_TYPE = 2,
    // Master interface B channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M00_B_REG_TYPE = 0,
    // Master interface AR channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M00_AR_REG_TYPE = 1,
    // Master interface R channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M00_R_REG_TYPE = 0,
    // Master interface AW channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M01_AW_REG_TYPE = 1,
    // Master interface W channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M01_W_REG_TYPE = 2,
    // Master interface B channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M01_B_REG_TYPE = 0,
    // Master interface AR channel register type (output)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M01_AR_REG_TYPE = 1,
    // Master interface R channel register type (input)
    // 0 to bypass, 1 for simple buffer, 2 for skid buffer
    parameter M01_R_REG_TYPE = 0
)
(
    input  wire                     clk,
    input  wire                     rst,

    /*
     * AXI lite slave interfaces
     */
    axil_intf.SLAVE                 s00_axil,

    axil_intf.SLAVE                 s01_axil,

    axil_intf.SLAVE                 s02_axil,

    axil_intf.SLAVE                 s03_axil,

    axil_intf.SLAVE                 s04_axil,

    /*
     * AXI lite master interfaces
     */
    axil_intf.MASTER                m00_axil,

    axil_intf.MASTER                m01_axil
);

localparam S_COUNT = 5;
localparam M_COUNT = 2;

// parameter sizing helpers
function [ADDR_WIDTH*M_REGIONS-1:0] w_a_r(input [ADDR_WIDTH*M_REGIONS-1:0] val);
    w_a_r = val;
endfunction

function [32*M_REGIONS-1:0] w_32_r(input [32*M_REGIONS-1:0] val);
    w_32_r = val;
endfunction

function [S_COUNT-1:0] w_s(input [S_COUNT-1:0] val);
    w_s = val;
endfunction

function [31:0] w_32(input [31:0] val);
    w_32 = val;
endfunction

function [1:0] w_2(input [1:0] val);
    w_2 = val;
endfunction

function w_1(input val);
    w_1 = val;
endfunction

axil_crossbar #(
    .S_COUNT(S_COUNT),
    .M_COUNT(M_COUNT),
    .DATA_WIDTH(DATA_WIDTH),
    .ADDR_WIDTH(ADDR_WIDTH),
    .STRB_WIDTH(STRB_WIDTH),
    .S_ACCEPT({ w_32(S04_ACCEPT), w_32(S03_ACCEPT), w_32(S02_ACCEPT), w_32(S01_ACCEPT), w_32(S00_ACCEPT) }),
    .M_REGIONS(M_REGIONS),
    .M_BASE_ADDR({ w_a_r(M01_BASE_ADDR), w_a_r(M00_BASE_ADDR) }),
    .M_ADDR_WIDTH({ w_32_r(M01_ADDR_WIDTH), w_32_r(M00_ADDR_WIDTH) }),
    .M_CONNECT_READ({ w_s(M01_CONNECT_READ), w_s(M00_CONNECT_READ) }),
    .M_CONNECT_WRITE({ w_s(M01_CONNECT_WRITE), w_s(M00_CONNECT_WRITE) }),
    .M_ISSUE({ w_32(M01_ISSUE), w_32(M00_ISSUE) }),
    .M_SECURE({ w_1(M01_SECURE), w_1(M00_SECURE) }),
    .S_AR_REG_TYPE({ w_2(S04_AR_REG_TYPE), w_2(S03_AR_REG_TYPE), w_2(S02_AR_REG_TYPE), w_2(S01_AR_REG_TYPE), w_2(S00_AR_REG_TYPE) }),
    .S_R_REG_TYPE({ w_2(S04_R_REG_TYPE), w_2(S03_R_REG_TYPE), w_2(S02_R_REG_TYPE), w_2(S01_R_REG_TYPE), w_2(S00_R_REG_TYPE) }),
    .S_AW_REG_TYPE({ w_2(S04_AW_REG_TYPE), w_2(S03_AW_REG_TYPE), w_2(S02_AW_REG_TYPE), w_2(S01_AW_REG_TYPE), w_2(S00_AW_REG_TYPE) }),
    .S_W_REG_TYPE({ w_2(S04_W_REG_TYPE), w_2(S03_W_REG_TYPE), w_2(S02_W_REG_TYPE), w_2(S01_W_REG_TYPE), w_2(S00_W_REG_TYPE) }),
    .S_B_REG_TYPE({ w_2(S04_B_REG_TYPE), w_2(S03_B_REG_TYPE), w_2(S02_B_REG_TYPE), w_2(S01_B_REG_TYPE), w_2(S00_B_REG_TYPE) }),
    .M_AR_REG_TYPE({ w_2(M01_AR_REG_TYPE), w_2(M00_AR_REG_TYPE) }),
    .M_R_REG_TYPE({ w_2(M01_R_REG_TYPE), w_2(M00_R_REG_TYPE) }),
    .M_AW_REG_TYPE({ w_2(M01_AW_REG_TYPE), w_2(M00_AW_REG_TYPE) }),
    .M_W_REG_TYPE({ w_2(M01_W_REG_TYPE), w_2(M00_W_REG_TYPE) }),
    .M_B_REG_TYPE({ w_2(M01_B_REG_TYPE), w_2(M00_B_REG_TYPE) })
)
axil_crossbar_inst (
    .clk(clk),
    .rst(rst),
    .s_axil_awaddr({ s04_axil.awaddr, s03_axil.awaddr, s02_axil.awaddr, s01_axil.awaddr, s00_axil.awaddr }),
    .s_axil_awprot({ s04_axil.awprot, s03_axil.awprot, s02_axil.awprot, s01_axil.awprot, s00_axil.awprot }),
    .s_axil_awvalid({ s04_axil.awvalid, s03_axil.awvalid, s02_axil.awvalid, s01_axil.awvalid, s00_axil.awvalid }),
    .s_axil_awready({ s04_axil.awready, s03_axil.awready, s02_axil.awready, s01_axil.awready, s00_axil.awready }),
    .s_axil_wdata({ s04_axil.wdata, s03_axil.wdata, s02_axil.wdata, s01_axil.wdata, s00_axil.wdata }),
    .s_axil_wstrb({ s04_axil.wstrb, s03_axil.wstrb, s02_axil.wstrb, s01_axil.wstrb, s00_axil.wstrb }),
    .s_axil_wvalid({ s04_axil.wvalid, s03_axil.wvalid, s02_axil.wvalid, s01_axil.wvalid, s00_axil.wvalid }),
    .s_axil_wready({ s04_axil.wready, s03_axil.wready, s02_axil.wready, s01_axil.wready, s00_axil.wready }),
    .s_axil_bresp({ s04_axil.bresp, s03_axil.bresp, s02_axil.bresp, s01_axil.bresp, s00_axil.bresp }),
    .s_axil_bvalid({ s04_axil.bvalid, s03_axil.bvalid, s02_axil.bvalid, s01_axil.bvalid, s00_axil.bvalid }),
    .s_axil_bready({ s04_axil.bready, s03_axil.bready, s02_axil.bready, s01_axil.bready, s00_axil.bready }),
    .s_axil_araddr({ s04_axil.araddr, s03_axil.araddr, s02_axil.araddr, s01_axil.araddr, s00_axil.araddr }),
    .s_axil_arprot({ s04_axil.arprot, s03_axil.arprot, s02_axil.arprot, s01_axil.arprot, s00_axil.arprot }),
    .s_axil_arvalid({ s04_axil.arvalid, s03_axil.arvalid, s02_axil.arvalid, s01_axil.arvalid, s00_axil.arvalid }),
    .s_axil_arready({ s04_axil.arready, s03_axil.arready, s02_axil.arready, s01_axil.arready, s00_axil.arready }),
    .s_axil_rdata({ s04_axil.rdata, s03_axil.rdata, s02_axil.rdata, s01_axil.rdata, s00_axil.rdata }),
    .s_axil_rresp({ s04_axil.rresp, s03_axil.rresp, s02_axil.rresp, s01_axil.rresp, s00_axil.rresp }),
    .s_axil_rvalid({ s04_axil.rvalid, s03_axil.rvalid, s02_axil.rvalid, s01_axil.rvalid, s00_axil.rvalid }),
    .s_axil_rready({ s04_axil.rready, s03_axil.rready, s02_axil.rready, s01_axil.rready, s00_axil.rready }),
    .m_axil_awaddr({ m01_axil.awaddr, m00_axil.awaddr }),
    .m_axil_awprot({ m01_axil.awprot, m00_axil.awprot }),
    .m_axil_awvalid({ m01_axil.awvalid, m00_axil.awvalid }),
    .m_axil_awready({ m01_axil.awready, m00_axil.awready }),
    .m_axil_wdata({ m01_axil.wdata, m00_axil.wdata }),
    .m_axil_wstrb({ m01_axil.wstrb, m00_axil.wstrb }),
    .m_axil_wvalid({ m01_axil.wvalid, m00_axil.wvalid }),
    .m_axil_wready({ m01_axil.wready, m00_axil.wready }),
    .m_axil_bresp({ m01_axil.bresp, m00_axil.bresp }),
    .m_axil_bvalid({ m01_axil.bvalid, m00_axil.bvalid }),
    .m_axil_bready({ m01_axil.bready, m00_axil.bready }),
    .m_axil_araddr({ m01_axil.araddr, m00_axil.araddr }),
    .m_axil_arprot({ m01_axil.arprot, m00_axil.arprot }),
    .m_axil_arvalid({ m01_axil.arvalid, m00_axil.arvalid }),
    .m_axil_arready({ m01_axil.arready, m00_axil.arready }),
    .m_axil_rdata({ m01_axil.rdata, m00_axil.rdata }),
    .m_axil_rresp({ m01_axil.rresp, m00_axil.rresp }),
    .m_axil_rvalid({ m01_axil.rvalid, m00_axil.rvalid }),
    .m_axil_rready({ m01_axil.rready, m00_axil.rready })
);

endmodule

`resetall
