module interconnect_top (
    input                   clk,
    input                   rst,

    axil_intf.SLAVE         s_axil_pcie,
    axil_intf.SLAVE         s_axil_vertex_gen,
    axil_intf.SLAVE         s_axil_primitive_processor,
    axil_intf.SLAVE         s_axil_rasterizer,

    axil_intf.SLAVE         s_axil_ctrl_vertex_gen,
    axil_intf.SLAVE         s_axil_ctrl_primitive_processor,
    axil_intf.SLAVE         s_axil_ctrl_rasterizer,

    axil_intf.MASTER        m_axil_ddr,

    axil_intf.MASTER        m_axil_cfg_pcie,
    axil_intf.MASTER        m_axil_cfg_vertex_gen,
    axil_intf.MASTER        m_axil_cfg_primitive_processor,
    axil_intf.MASTER        m_axil_cfg_rasterizer
);

axil_intf #(.AXIL_DATA_WIDTH(128), .AXIL_ADDR_WIDTH(32)) cfg_axil();
axil_intf #(.AXIL_DATA_WIDTH(32), .AXIL_ADDR_WIDTH(32))  cfg_axil_downconvert();

axil_intf #(.AXIL_DATA_WIDTH(32), .AXIL_ADDR_WIDTH(32))  ctrl_axil();
axil_intf #(.AXIL_DATA_WIDTH(32), .AXIL_ADDR_WIDTH(32))  ctrl_axil_dummy();
axil_intf #(.AXIL_DATA_WIDTH(128), .AXIL_ADDR_WIDTH(32)) ctrl_axil_upconvert();

axil_crossbar_wrap_3x2 #(
    .DATA_WIDTH             (32),
    .ADDR_WIDTH             (32),
    
    .M00_BASE_ADDR          (32'h00000000),     // access to ddr
    .M00_ADDR_WIDTH         (31),

    .M01_BASE_ADDR          (32'h80000000),     // access to config (blocked)
    .M01_ADDR_WIDTH         (31),
    .M01_CONNECT_READ       (3'b0),
    .M01_CONNECT_WRITE      (3'b0)
) u_ctrl_xbar (
    .clk                    (clk),
    .rst                    (rst),

    .s00_axil               (s_axil_ctrl_vertex_gen),
    .s01_axil               (s_axil_ctrl_primitive_processor),
    .s02_axil               (s_axil_ctrl_rasterizer),

    .m00_axil               (ctrl_axil),
    .m01_axil               (ctrl_axil_dummy)
);

axil_adapter_wrapper u_ctrl_convert (
    .clk                    (clk),
    .rst                    (rst),

    .s_axil                 (ctrl_axil),
    .m_axil                 (ctrl_axil_upconvert)
);


axil_crossbar_wrap_5x2 #(
    .DATA_WIDTH             (128),
    .ADDR_WIDTH             (32),

    // ddr master
    .M00_BASE_ADDR          (32'h00000000),     // 0x0 -> 0x7fffffff
    .M00_ADDR_WIDTH         (31),
    .M00_CONNECT_READ       (5'b11111),         // all masters
    .M00_CONNECT_WRITE      (5'b11111),         // all masters

    // configuration master
    .M01_BASE_ADDR          (32'h80000000),     // 0x80000000 -> 0xffffffff
    .M01_ADDR_WIDTH         (31),
    .M01_CONNECT_READ       (5'b00001),         // only pcie master
    .M01_CONNECT_WRITE      (5'b00001)          // only pcie master
) u_data_xbar (
    .clk                    (clk),
    .rst                    (rst),

    .s00_axil               (s_axil_pcie),
    .s01_axil               (s_axil_vertex_gen),
    .s02_axil               (s_axil_primitive_processor),
    .s03_axil               (s_axil_rasterizer),
    .s04_axil               (ctrl_axil_upconvert),

    .m00_axil               (m_axil_ddr),
    .m01_axil               (cfg_axil)
);

axil_adapter_wrapper u_cfg_convert (
    .clk                    (clk),
    .rst                    (rst),

    .s_axil                 (cfg_axil),
    .m_axil                 (cfg_axil_downconvert)
);

axil_interconnect_wrap_1x4 #(
    .DATA_WIDTH             (32),
    .ADDR_WIDTH             (32),

    .M00_BASE_ADDR          (32'h80000000),
    .M00_ADDR_WIDTH         (16),
    .M01_BASE_ADDR          (32'h80010000),
    .M01_ADDR_WIDTH         (16),
    .M02_BASE_ADDR          (32'h80020000),
    .M02_ADDR_WIDTH         (16),
    .M03_BASE_ADDR          (32'h80030000),
    .M03_ADDR_WIDTH         (16)
) u_cfg_interconnect (
    .clk                    (clk),
    .rst                    (rst),

    .s00_axil               (cfg_axil_downconvert),
    
    .m00_axil               (m_axil_cfg_pcie),
    .m01_axil               (m_axil_cfg_vertex_gen),
    .m02_axil               (m_axil_cfg_primitive_processor),
    .m03_axil               (m_axil_cfg_rasterizer)
);

endmodule