import rtl_manifest
import os


all_sources = rtl_manifest.read_sources("../../sources.list")

verilog_sources = list(filter(lambda s: (s.endswith(".v") or s.endswith(".sv")), all_sources))
xci_sources = list(filter(lambda s: s.endswith(".xci"), all_sources))
xdc_sources = list(filter(lambda s: s.endswith(".xdc"), all_sources))
tcl_sources = list(filter(lambda s: s.endswith(".tcl"), all_sources))

part = "xc7a200tfbg484-2"
top = "vertex_generator_datapath"

synth_directive = "PerformanceOptimized"


with open("build.tcl", "w") as f:
    f.write(f"set outputDir ./datapath_synthesis\n")
    f.write(f"file mkdir $outputDir\n")

    f.write(f"set_part {part}\n")

    f.write(f"read_verilog -sv {' '.join(verilog_sources)}\n")
    
    f.write(f"synth_design -top {top} -directive {synth_directive}\n")
    f.write(f"write_checkpoint -force $outputDir/post_synth.dcp\n")
    f.write(f"report_utilization -hierarchical -file $outputDir/post_synth_util.rpt\n")
    f.write(f"opt_design\n")
    f.write(f"report_utilization -file $outputDir/post_optimization_util.rpt\n")
    f.write(f"exit\n")


os.system(f"vivado -nolog -nojournal -mode tcl -source build.tcl")
os.remove("build.tcl")
