import fp32_pkg::*;

module vertex_generator_datapath_harness();

`include "fp32_helpers.svh"

logic clk;

logic i_valid;

fp32_t world_point[3];

fp32_t world_to_camera_matrix[4][4];
fp32_t projection_matrix[4][4];
fp32_t screen_dim[2];
fp32_t raster_points[3];

logic o_visible;
logic o_valid;

`FP32_INPUT(world0);
`FP32_INPUT(world1);
`FP32_INPUT(world2);

`FP32_INPUT(world_to_camera_matrix00);
`FP32_INPUT(world_to_camera_matrix01);
`FP32_INPUT(world_to_camera_matrix02);
`FP32_INPUT(world_to_camera_matrix03);
`FP32_INPUT(world_to_camera_matrix10);
`FP32_INPUT(world_to_camera_matrix11);
`FP32_INPUT(world_to_camera_matrix12);
`FP32_INPUT(world_to_camera_matrix13);
`FP32_INPUT(world_to_camera_matrix20);
`FP32_INPUT(world_to_camera_matrix21);
`FP32_INPUT(world_to_camera_matrix22);
`FP32_INPUT(world_to_camera_matrix23);
`FP32_INPUT(world_to_camera_matrix30);
`FP32_INPUT(world_to_camera_matrix31);
`FP32_INPUT(world_to_camera_matrix32);
`FP32_INPUT(world_to_camera_matrix33);

`FP32_INPUT(projection_matrix00);
`FP32_INPUT(projection_matrix01);
`FP32_INPUT(projection_matrix02);
`FP32_INPUT(projection_matrix03);
`FP32_INPUT(projection_matrix10);
`FP32_INPUT(projection_matrix11);
`FP32_INPUT(projection_matrix12);
`FP32_INPUT(projection_matrix13);
`FP32_INPUT(projection_matrix20);
`FP32_INPUT(projection_matrix21);
`FP32_INPUT(projection_matrix22);
`FP32_INPUT(projection_matrix23);
`FP32_INPUT(projection_matrix30);
`FP32_INPUT(projection_matrix31);
`FP32_INPUT(projection_matrix32);
`FP32_INPUT(projection_matrix33);

`FP32_INPUT(screen_x);
`FP32_INPUT(screen_y);

`FP32_OUTPUT(raster0);
`FP32_OUTPUT(raster1);
`FP32_OUTPUT(raster2);

assign world_point = {world0, world1, world2};

assign world_to_camera_matrix = 
  ({{world_to_camera_matrix00, world_to_camera_matrix01, world_to_camera_matrix02, world_to_camera_matrix03},
    {world_to_camera_matrix10, world_to_camera_matrix11, world_to_camera_matrix12, world_to_camera_matrix13},
    {world_to_camera_matrix20, world_to_camera_matrix21, world_to_camera_matrix22, world_to_camera_matrix23},
    {world_to_camera_matrix30, world_to_camera_matrix31, world_to_camera_matrix32, world_to_camera_matrix33}
   });

assign projection_matrix = 
  ({{projection_matrix00, projection_matrix01, projection_matrix02, projection_matrix03},
    {projection_matrix10, projection_matrix11, projection_matrix12, projection_matrix13},
    {projection_matrix20, projection_matrix21, projection_matrix22, projection_matrix23},
    {projection_matrix30, projection_matrix31, projection_matrix32, projection_matrix33}
   });

assign screen_dim = {screen_x, screen_y};

assign raster0 = raster_points[0];
assign raster1 = raster_points[1];
assign raster2 = raster_points[2];

vertex_generator_datapath u_dut(
    .clk                    (clk),
    .i_valid                (i_valid),

    .world_point            (world_point),
    .world_to_camera_matrix (world_to_camera_matrix),
    .projection_matrix      (projection_matrix),
    .screen_dim             (screen_dim),

    .raster_points          (raster_points),
    .o_visible              (o_visible),
    .o_valid                (o_valid)
);

endmodule