// this module implements the "compute_pixel_coordinates" function in the
// python model
import fp32_pkg::*;

module vertex_generator_datapath(
    input clk,

    input i_valid,

    input fp32_t world_point[3],

    input fp32_t world_to_camera_matrix[4][4],
    input fp32_t projection_matrix[4][4],

    input fp32_t screen_dim [2],

    output fp32_t raster_points[3], // the z coordinate is not raster, but x and y are
    output logic o_visible,

    output logic o_valid
);


logic camera_matmul_valid;

fp32_t camera_point [4];

fp32_vec4mat44mul u_camera_point_matmul (
    .clk        (clk),
    .i_valid    (i_valid),
    .v          ({world_point[0], world_point[1], world_point[2], `FP32_ONE}),
    .a          (world_to_camera_matrix),
    .y          (camera_point),
    .o_valid    (camera_matmul_valid)
);

logic projection_matmul_valid;

fp32_t projected [4];

fp32_vec4mat44mul u_projection_matmul (
    .clk        (clk),
    .i_valid    (camera_matmul_valid),
    .v          (camera_point),
    .a          (projection_matrix),
    .y          (projected),
    .o_valid    (projection_matmul_valid)
);

logic normalization_valid;

fp32_t normalized [3];
logic normalized_visible;

vertex_generator_normalization u_normalization (
    .clk        (clk),
    .i_valid    (projection_matmul_valid),
    .v          (projected),
    .y          (normalized),
    .o_visible  (normalized_visible),
    .o_valid    (normalization_valid)
);

vertex_generator_raster_coordinate_gen u_raster_coordinate_generation (
    .clk        (clk),
    .i_valid    (normalization_valid),
    .screen_dim (screen_dim),
    .i_visible  (normalized_visible),
    .v          (normalized),
    .y          (raster_points),
    .o_visible  (o_visible),
    .o_valid    (o_valid)
);

endmodule