import fp32_pkg::*;

module vertex_generator_normalization(
    input clk,

    input i_valid,

    input fp32_t v[4],

    output fp32_t y[3],
    output logic o_visible,

    output o_valid
);

fp32_t div_result[3];

logic [2:0] div_valid;

generate
    for (genvar i = 0; i < 3; i++) begin : coordinate_div
        fp32_div u_div_x(
            .clk        (clk),
            .i_valid    (i_valid),
            .a          (v[i]),
            .b          (v[3]),
            .z          (div_result[i]),
            .o_valid    (div_valid[i])
        );
    end
endgenerate

always_ff @(posedge clk) begin
    for (int i = 0; i < 3; i++) begin
        y[i] <= div_result[i];
    end
end

fp32_t negative_1;
fp32_t positive_1;

assign negative_1.sign = 1;
assign negative_1.exponent = fp32_exp_bias;
assign negative_1.mantissa = '0;

assign positive_1.sign = '0;
assign positive_1.exponent = fp32_exp_bias;
assign positive_1.mantissa = '0;

logic [1:0] bounds_valid;
logic [1:0] coordinate_visible;


assign o_valid = &bounds_valid;
assign o_visible = &coordinate_visible;

generate
    for (genvar i = 0; i < 2; i++) begin : visibility_check
        logic lower_bound;
        logic upper_bound;

        logic lower_bound_valid;
        logic upper_bound_valid;

        fp32_gt u_lower_bound_check (
            .clk        (clk),
            .i_valid    (div_valid[i]),
            .a          (div_result[i]),
            .b          (negative_1),
            .gt         (lower_bound),
            .o_valid    (lower_bound_valid)
        );

        fp32_gt u_upper_bound_check (
            .clk        (clk),
            .i_valid    (div_valid[i]),
            .a          (positive_1),
            .b          (div_result[i]),
            .gt         (upper_bound),
            .o_valid    (upper_bound_valid)
        );

        assign bounds_valid[i] = lower_bound_valid & upper_bound_valid;
        assign coordinate_visible[i] = lower_bound & upper_bound;
    end
endgenerate

endmodule