import fp32_pkg::*;


module vertex_generator_raster_coordinate_gen(
    input clk,

    input i_valid,

    input fp32_t screen_dim[2],

    input fp32_t v[3],
    input i_visible,

    output fp32_t y[3],

    output logic o_visible,
    output logic o_valid
);

//  raster_x = (projected[0] + 1)/2 * image_width
//  raster_y = (projected[1] + 1)/2 * image_height
//  z coordinate is passed through unchanged
//  visible flag is passed through unchanged

localparam N_STAGES = 6;

fp32_t z_sr[N_STAGES];
logic visible_sr[N_STAGES];

always @(posedge clk) begin
    for (int i = 0; i < N_STAGES-1; i++) begin
        z_sr[i+1] <= z_sr[i];
        visible_sr[i+1] <= visible_sr[i];
    end

    z_sr[0] <= v[2];
    visible_sr[0] <= i_visible;
end


assign y[2] = z_sr[N_STAGES-1];
assign o_visible = visible_sr[N_STAGES-1];


logic [1:0] valid;

assign o_valid = &valid;

generate
    for (genvar i = 0; i < 2; i++) begin : coordinate_loop
        fp32_t sum;
        logic sum_valid;

        fp32_t quotient;
        logic quotient_valid;

        fp32_t product;
        logic product_valid;

        fp32_add u_add (
            .clk        (clk),
            .i_valid    (i_valid),
            .a          (v[i]),
            .b          (`FP32_ONE),
            .z          (sum),
            .o_valid    (sum_valid)
        );

        // optionally register this
        assign quotient.sign = sum.sign;
        assign quotient.exponent = sum.exponent - 1;    // easy divide by 2
        assign quotient.mantissa = sum.mantissa;
        assign quotient_valid = sum_valid;

        fp32_mul u_mul (
            .clk        (clk),
            .i_valid    (quotient_valid),
            .a          (quotient),
            .b          (screen_dim[i]),
            .z          (y[i]),
            .o_valid    (product_valid)
        );

        assign valid[i] = product_valid;
    end
endgenerate



endmodule