module vertex_generator_read_fsm(
    input               clk,
    input               rst,

    input logic [31:0]  source_addr,
    input logic [31:0]  len,
    input logic         start,

    output logic        done,

    output logic        valid,
    input               ready,
    output fp32_t       vertex [3],

    axil_intf.MASTER    m_axil_read
);


logic [95:0] data_buffer, data_buffer_next;

logic [31:0] rd_addr, rd_addr_next;

logic [31:0] output_count, output_count_next;

logic active, active_next;

enum logic [3:0] {
    ZERO,
    ONE,
    TWO,
    THREE,
    IDLE
} state, state_next;

always_ff @(posedge clk) begin
    if (rst) begin
        state <= IDLE;
    end else begin
        state <= state_next;
        rd_addr <= rd_addr_next;
        active <= active_next;
        data_buffer <= data_buffer_next;
        output_count <= output_count_next;
    end
end

always_comb begin
    active_next = active;

    state_next = state;

    data_buffer_next = data_buffer;
    rd_addr_next = rd_addr;

    m_axil_read.rready = '1;
    m_axil_read.araddr = rd_addr;

    output_count_next = output_count;

    if (start) begin
        rd_addr_next = source_addr;
        output_count_next = '0;
        active_next = '1;
    end

    done = '0;


    if (active & ready) begin
        m_axil_read.arvalid = '1;

        if (m_axil_read.arready) begin
            rd_addr_next = rd_addr + 16;

            if (rd_addr_next > source_addr + (len*16)) begin
                active_next = '0;
            end
        end
    end else begin
        m_axil_read.arvalid = '0;
    end

    vertex = '{default: '0};
    valid = '0;

    case (state)
        IDLE: begin
            done = '1;

            if (start) begin
                state_next = ZERO;
            end
        end

        ZERO: begin
            m_axil_read.rready = ready;
            if (m_axil_read.rvalid & ready) begin
                vertex = {m_axil_read.rdata[31:0], m_axil_read.rdata[63:32], m_axil_read.rdata[95:64]};
                data_buffer_next[31:0] = m_axil_read.rdata[127:96];
                valid = '1;

                state_next = ONE;

                output_count_next = output_count + 1;

                if (output_count_next == len) begin
                    state_next = IDLE;
                end
            end
        end

        ONE: begin
            m_axil_read.rready = ready;
            if (m_axil_read.rvalid & ready) begin
                vertex = {data_buffer[31:0], m_axil_read.rdata[31:0], m_axil_read.rdata[63:32]};
                data_buffer_next[63:0] = m_axil_read.rdata[127:64];
                valid = '1;

                state_next = TWO;

                output_count_next = output_count + 1;

                if (output_count_next == len) begin
                    state_next = IDLE;
                end
            end
        end

        TWO: begin
            m_axil_read.rready = ready;
            if (m_axil_read.rvalid & ready) begin
                vertex = {data_buffer[31:0], data_buffer[63:32], m_axil_read.rdata[31:0]};
                data_buffer_next[95:0] = m_axil_read.rdata[127:32];
                valid = '1;

                state_next = THREE;

                output_count_next = output_count + 1;

                if (output_count_next == len) begin
                    state_next = IDLE;
                end
            end
        end

        THREE: begin
            m_axil_read.rready = '0;
            vertex = {data_buffer[31:0], data_buffer[63:32], data_buffer[95:64]};
            valid = '1;

            if (ready) begin
                state_next = ZERO;
                output_count_next = output_count + 1;

                if (output_count_next == len) begin
                    state_next = IDLE;
                end
            end
        end

        default: begin
            state_next = IDLE;
        end
    endcase
end



endmodule