module vertex_generator_write_fsm(
    input               clk,
    input               rst,

    input logic [31:0]  dest_addr,
    input logic [31:0]  len,
    input logic         start,

    output logic        done,

    output logic        ready,

    input logic         valid,
    input fp32_t        vertex [3],
    input logic         visible,

    axil_intf.MASTER    m_axil_write
);

logic [95:0] data_buffer, data_buffer_next;

logic [31:0] wr_addr, wr_addr_next;

logic [31:0] output_count, output_count_next;

logic active, active_next;

enum logic [0:0] {
    IDLE,
    RUNNING
} state, state_next;

always_ff @(posedge clk) begin
    if (rst) begin
        state <= IDLE;
    end else begin
        state <= state_next;

        active <= active_next;
        data_buffer <= data_buffer_next;
        wr_addr <= wr_addr_next;
        output_count <= output_count_next;
    end
end


always_comb begin
    active_next = active;

    state_next = state;

    data_buffer_next = data_buffer;
    wr_addr_next = wr_addr;

    m_axil_write.bready = '1;
    m_axil_write.awaddr = wr_addr;

    m_axil_write.wstrb = '1;

    output_count_next = output_count;

    if (start) begin
        wr_addr_next = dest_addr;
        output_count_next = '0;
        active_next = '1;
    end

    done = '0;


    if (active) begin
        m_axil_write.awvalid = '1;

        if (m_axil_write.awready) begin
            wr_addr_next = wr_addr + 16;

            if (wr_addr_next >= dest_addr + (len*16)) begin
                active_next = '0;
            end
        end
    end else begin
        m_axil_write.awvalid = '0;
    end

    m_axil_write.wvalid = '0;
    m_axil_write.wdata = '0;

    ready = '0;


    case (state)
        IDLE: begin
            done = '1;
            ready = '0;

            if (start) begin
                state_next = RUNNING;
            end
        end

        RUNNING: begin
            m_axil_write.wvalid = valid;
            m_axil_write.wstrb = '1;
            ready = m_axil_write.wready;

            if (valid & ready) begin
                m_axil_write.wdata = {31'b0, visible, vertex[2], vertex[1], vertex[0]};

                output_count_next = output_count + 1;

                if (output_count_next == len) begin
                    state_next = IDLE;
                end
            end
        end

        default: begin
            state_next = IDLE;
        end

    endcase
end

endmodule