module mig_wrapper (
    inout [31:0]            ddr3_dq,
    inout [3:0]             ddr3_dqs_n,
    inout [3:0]             ddr3_dqs_p,

    output [14:0]           ddr3_addr,
    output [2:0]            ddr3_ba,
    output                  ddr3_ras_n,
    output                  ddr3_cas_n,
    output                  ddr3_we_n,
    output                  ddr3_reset_n,
    output [0:0]            ddr3_ck_p,
    output [0:0]            ddr3_ck_n,
    output [0:0]            ddr3_cke,
    output [0:0]            ddr3_cs_n,
    output [3:0]            ddr3_dm,
    output [0:0]            ddr3_odt,

    output                  init_calib_complete,

    input                   clk_100,

    input                   aresetn,
    input                   sys_clk,

    axil_intf.SLAVE          s_axil
);


logic ui_clk;
logic ui_rst;

axil_intf #(.AXIL_DATA_WIDTH(128), .AXIL_ADDR_WIDTH(32)) s_axil_cdc();


axil_cdc_wrapper u_axil_cdc_wrapper (
    .s_clk      (clk_100),
    .s_rst      (~aresetn),
    .s_axil     (s_axil),

    .m_clk      (ui_clk),
    .m_rst      (ui_rst),
    .m_axil     (s_axil_cdc)
);

`ifdef DEBUG_ILA
ila_0 ddr_axil_ila (
	.clk(ui_clk), // input wire clk


	.probe0(    s_axil_cdc.wready), // input wire [0:0] probe0  
	.probe1(    s_axil_cdc.awaddr), // input wire [31:0]  probe1 
	.probe2(    s_axil_cdc.bresp), // input wire [1:0]  probe2 
	.probe3(    s_axil_cdc.bvalid), // input wire [0:0]  probe3 
	.probe4(    s_axil_cdc.bready), // input wire [0:0]  probe4 
	.probe5(    s_axil_cdc.araddr), // input wire [31:0]  probe5 
	.probe6(    s_axil_cdc.rready), // input wire [0:0]  probe6 
	.probe7(    s_axil_cdc.wvalid), // input wire [0:0]  probe7 
	.probe8(    s_axil_cdc.arvalid), // input wire [0:0]  probe8 
	.probe9(    s_axil_cdc.arready), // input wire [0:0]  probe9 
	.probe10(   s_axil_cdc.rdata), // input wire [127:0]  probe10 
	.probe11(   s_axil_cdc.awvalid), // input wire [0:0]  probe11 
	.probe12(   s_axil_cdc.awready), // input wire [0:0]  probe12 
	.probe13(   s_axil_cdc.rresp), // input wire [1:0]  probe13 
	.probe14(   s_axil_cdc.wdata), // input wire [127:0]  probe14 
	.probe15(   s_axil_cdc.wstrb), // input wire [15:0]  probe15 
	.probe16(   s_axil_cdc.rvalid), // input wire [0:0]  probe16 
	.probe17(   s_axil_cdc.arprot), // input wire [2:0]  probe17  
	.probe18(   s_axil_cdc.awprot) // input wire [2:0]  probe18
);
`endif

ddr3 u_ddr3 (

    // Memory interface ports
    .ddr3_addr                      (ddr3_addr),  // output [14:0]		ddr3_addr
    .ddr3_ba                        (ddr3_ba),  // output [2:0]		ddr3_ba
    .ddr3_cas_n                     (ddr3_cas_n),  // output			ddr3_cas_n
    .ddr3_ck_n                      (ddr3_ck_n),  // output [0:0]		ddr3_ck_n
    .ddr3_ck_p                      (ddr3_ck_p),  // output [0:0]		ddr3_ck_p
    .ddr3_cke                       (ddr3_cke),  // output [0:0]		ddr3_cke
    .ddr3_ras_n                     (ddr3_ras_n),  // output			ddr3_ras_n
    .ddr3_reset_n                   (ddr3_reset_n),  // output			ddr3_reset_n
    .ddr3_we_n                      (ddr3_we_n),  // output			ddr3_we_n
    .ddr3_dq                        (ddr3_dq),  // inout [31:0]		ddr3_dq
    .ddr3_dqs_n                     (ddr3_dqs_n),  // inout [3:0]		ddr3_dqs_n
    .ddr3_dqs_p                     (ddr3_dqs_p),  // inout [3:0]		ddr3_dqs_p
    .init_calib_complete            (init_calib_complete),  // output			init_calib_complete
      
	.ddr3_cs_n                      (ddr3_cs_n),  // output [0:0]		ddr3_cs_n
    .ddr3_dm                        (ddr3_dm),  // output [3:0]		ddr3_dm
    .ddr3_odt                       (ddr3_odt),  // output [0:0]		ddr3_odt
    // Application interface ports
    .ui_clk                         (ui_clk),  // output			ui_clk
    .ui_clk_sync_rst                (ui_rst),  // output			ui_clk_sync_rst
    .mmcm_locked                    (),  // output			mmcm_locked
    .aresetn                        (aresetn),  // input			aresetn
    .app_sr_req                     ('0),  // input			app_sr_req
    .app_ref_req                    ('0),  // input			app_ref_req
    .app_zq_req                     ('0),  // input			app_zq_req
    .app_sr_active                  (),  // output			app_sr_active
    .app_ref_ack                    (),  // output			app_ref_ack
    .app_zq_ack                     (),  // output			app_zq_ack
    // Slave Interface Write Address Ports
    .s_axi_awid                     ('0),  // input [3:0]			s_axi_awid
    .s_axi_awaddr                   (s_axil_cdc.awaddr[29:0]),  // input [29:0]			s_axi_awaddr
    .s_axi_awlen                    ('0),  // input [7:0]			s_axi_awlen
    .s_axi_awsize                   (3'b100),  // input [2:0]			s_axi_awsize
    .s_axi_awburst                  (2'b01),  // input [1:0]			s_axi_awburst
    .s_axi_awlock                   ('0),  // input [0:0]			s_axi_awlock
    .s_axi_awcache                  (4'h3),  // input [3:0]			s_axi_awcache
    .s_axi_awprot                   ('0),  // input [2:0]			s_axi_awprot
    .s_axi_awqos                    ('0),  // input [3:0]			s_axi_awqos
    .s_axi_awvalid                  (s_axil_cdc.awvalid),  // input			s_axi_awvalid
    .s_axi_awready                  (s_axil_cdc.awready),  // output			s_axi_awready
    // Slave Interface Write Data Ports
    .s_axi_wdata                    (s_axil_cdc.wdata),  // input [127:0]			s_axi_wdata
    .s_axi_wstrb                    (s_axil_cdc.wstrb),  // input [15:0]			s_axi_wstrb
    .s_axi_wlast                    ('1),  // input			s_axi_wlast
    .s_axi_wvalid                   (s_axil_cdc.wvalid),  // input			s_axi_wvalid
    .s_axi_wready                   (s_axil_cdc.wready),  // output			s_axi_wready
    // Slave Interface Write Response Ports
    .s_axi_bid                      (),  // output [3:0]			s_axi_bid
    .s_axi_bresp                    (s_axil_cdc.bresp),  // output [1:0]			s_axi_bresp
    .s_axi_bvalid                   (s_axil_cdc.bvalid),  // output			s_axi_bvalid
    .s_axi_bready                   (s_axil_cdc.bready),  // input			s_axi_bready
    // Slave Interface Read Address Ports
    .s_axi_arid                     ('0),  // input [3:0]			s_axi_arid
    .s_axi_araddr                   (s_axil_cdc.araddr[29:0]),  // input [29:0]			s_axi_araddr
    .s_axi_arlen                    ('0),  // input [7:0]			s_axi_arlen
    .s_axi_arsize                   (3'b100),  // input [2:0]			s_axi_arsize
    .s_axi_arburst                  (2'b01),  // input [1:0]			s_axi_arburst
    .s_axi_arlock                   ('0),  // input [0:0]			s_axi_arlock
    .s_axi_arcache                  (4'h3),  // input [3:0]			s_axi_arcache
    .s_axi_arprot                   ('0),  // input [2:0]			s_axi_arprot
    .s_axi_arqos                    ('0),  // input [3:0]			s_axi_arqos
    .s_axi_arvalid                  (s_axil_cdc.arvalid),  // input			s_axi_arvalid
    .s_axi_arready                  (s_axil_cdc.arready),  // output			s_axi_arready
    // Slave Interface Read Data Ports
    .s_axi_rid                      (),  // output [3:0]			s_axi_rid
    .s_axi_rdata                    (s_axil_cdc.rdata),  // output [127:0]			s_axi_rdata
    .s_axi_rresp                    (s_axil_cdc.rresp),  // output [1:0]			s_axi_rresp
    .s_axi_rlast                    (),  // output			s_axi_rlast
    .s_axi_rvalid                   (s_axil_cdc.rvalid),  // output			s_axi_rvalid
    .s_axi_rready                   (s_axil_cdc.rready),  // input			s_axi_rready
`ifdef VERILATOR
    .device_temp                    (),
`endif
    // System Clock Ports
    .sys_clk_i                      (sys_clk),
    .sys_rst                        (aresetn) // input sys_rst
);

endmodule