module pcie_wrapper (
    input  logic clk_pcie,

    input  logic rst_100,
    input rstn,
    input  logic clk_100,

    output logic link_up,

    axil_intf.MASTER m_data_axil,
    axil_intf.SLAVE  s_ctrl_axil,

    output wire [1 : 0] pci_exp_txp,
    output wire [1 : 0] pci_exp_txn,
    input wire [1 : 0] pci_exp_rxp,
    input wire [1 : 0] pci_exp_rxn
);

logic clk_axi;
logic rst_axi;

axi_intf #(.AXI_DATA_WIDTH(64), .AXI_ADDR_WIDTH(32), .AXI_ID_WIDTH(4)) m_axi();
axi_intf #(.AXI_DATA_WIDTH(64), .AXI_ADDR_WIDTH(32), .AXI_ID_WIDTH(4)) s_axi();

axil_intf #(.AXIL_DATA_WIDTH(128), .AXIL_ADDR_WIDTH(32)) m_axil_125();
axil_intf #(.AXIL_DATA_WIDTH(32), .AXIL_ADDR_WIDTH(32)) s_axi_ctl();

assign s_axi.awid = '0;
assign s_axi.awaddr = '0;
assign s_axi.awregion = '0;
assign s_axi.awlen = '0;
assign s_axi.awsize = '0;
assign s_axi.awburst = '0;
assign s_axi.awvalid = '0;
assign s_axi.wdata = '0;
assign s_axi.wstrb = '0;
assign s_axi.wlast = '0;
assign s_axi.wvalid = '0;
assign s_axi.bready = '0;
assign s_axi.arid = '0;
assign s_axi.araddr = '0;
assign s_axi.arregion = '0;
assign s_axi.arlen = '0;
assign s_axi.arsize = '0;
assign s_axi.arburst = '0;
assign s_axi.arvalid = '0;
assign s_axi.rready = '0;

axil_cdc_wrapper u_data_axil_cdc_wrapper (
    .s_clk              (clk_axi),
    .s_rst              (rst_axi),
    .s_axil             (m_axil_125),

    .m_clk              (clk_100),
    .m_rst              (rst_100),
    .m_axil             (m_data_axil)
);

axil_cdc_wrapper u_ctrl_axil_cdc_wrapper (
    .s_clk              (clk_100),
    .s_rst              (rst_100),
    .s_axil             (s_ctrl_axil),

    .m_clk              (clk_axi),
    .m_rst              (rst_axi),
    .m_axil             (s_axi_ctl)
);

`ifdef DEBUG_ILA
ila_1 axi_ila (
	.clk(clk_axi), // input wire clk


	.probe0(    m_axi.wready), // input wire [0:0] probe0  
	.probe1(    m_axi.awaddr), // input wire [31:0]  probe1 
	.probe2(    m_axi.bresp), // input wire [1:0]  probe2 
	.probe3(    m_axi.bvalid), // input wire [0:0]  probe3 
	.probe4(    m_axi.bready), // input wire [0:0]  probe4 
	.probe5(    m_axi.araddr), // input wire [31:0]  probe5 
	.probe6(    m_axi.rready), // input wire [0:0]  probe6 
	.probe7(    m_axi.wvalid), // input wire [0:0]  probe7 
	.probe8(    m_axi.arvalid), // input wire [0:0]  probe8 
	.probe9(    m_axi.arready), // input wire [0:0]  probe9 
	.probe10(   m_axi.rdata), // input wire [63:0]  probe10 
	.probe11(   m_axi.awvalid), // input wire [0:0]  probe11 
	.probe12(   m_axi.awready), // input wire [0:0]  probe12 
	.probe13(   m_axi.rresp), // input wire [1:0]  probe13 
	.probe14(   m_axi.wdata), // input wire [63:0]  probe14 
	.probe15(   m_axi.wstrb), // input wire [7:0]  probe15 
	.probe16(   m_axi.rvalid), // input wire [0:0]  probe16 
	.probe17(   m_axi.arprot), // input wire [2:0]  probe17 
	.probe18(   m_axi.awprot), // input wire [2:0]  probe18 
	.probe19(   m_axi.awid), // input wire [0:0]  probe19 
	.probe20(   m_axi.bid), // input wire [0:0]  probe20 
	.probe21(   m_axi.awlen), // input wire [7:0]  probe21 
	.probe22(   m_axi.buser), // input wire [0:0]  probe22 
	.probe23(   m_axi.awsize), // input wire [2:0]  probe23 
	.probe24(   m_axi.awburst), // input wire [1:0]  probe24 
	.probe25(   m_axi.arid), // input wire [0:0]  probe25 
	.probe26(   m_axi.awlock), // input wire [0:0]  probe26 
	.probe27(   m_axi.arlen), // input wire [7:0]  probe27 
	.probe28(   m_axi.arsize), // input wire [2:0]  probe28 
	.probe29(   m_axi.arburst), // input wire [1:0]  probe29 
	.probe30(   m_axi.arlock), // input wire [0:0]  probe30 
	.probe31(   m_axi.arcache), // input wire [3:0]  probe31 
	.probe32(   m_axi.awcache), // input wire [3:0]  probe32 
	.probe33(   m_axi.arregion), // input wire [3:0]  probe33 
	.probe34(   m_axi.arqos), // input wire [3:0]  probe34 
	.probe35(   m_axi.aruser), // input wire [0:0]  probe35 
	.probe36(   m_axi.awregion), // input wire [3:0]  probe36 
	.probe37(   m_axi.awqos), // input wire [3:0]  probe37 
	.probe38(   m_axi.rid), // input wire [0:0]  probe38 
	.probe39(   m_axi.awuser), // input wire [0:0]  probe39 
	.probe40(  '0), // input wire [0:0]  probe40 
	.probe41(   m_axi.rlast), // input wire [0:0]  probe41 
	.probe42(   m_axi.ruser), // input wire [0:0]  probe42  
	.probe43(   m_axi.wlast) // input wire [0:0]  probe43
);
`endif

axi_axil_adapter #(
  .AXI_DATA_WIDTH(64),
  .AXIL_DATA_WIDTH(128),
  .AXI_ID_WIDTH(4)
) u_axi_axil_adapter (
    .clk                (clk_axi),
    .rst                (rst_axi),

    .s_axi_awid         (m_axi.awid),
    .s_axi_awaddr       (m_axi.awaddr),
    .s_axi_awlen        (m_axi.awlen),
    .s_axi_awsize       (m_axi.awsize),
    .s_axi_awburst      (m_axi.awburst),
    .s_axi_awlock       (m_axi.awlock),
    .s_axi_awcache      (m_axi.awcache),
    .s_axi_awprot       (m_axi.awprot),
    .s_axi_awvalid      (m_axi.awvalid),
    .s_axi_awready      (m_axi.awready),
    .s_axi_wdata        (m_axi.wdata),
    .s_axi_wstrb        (m_axi.wstrb),
    .s_axi_wlast        (m_axi.wlast),
    .s_axi_wvalid       (m_axi.wvalid),
    .s_axi_wready       (m_axi.wready),
    .s_axi_bid          (m_axi.bid),
    .s_axi_bresp        (m_axi.bresp),
    .s_axi_bvalid       (m_axi.bvalid),
    .s_axi_bready       (m_axi.bready),
    .s_axi_arid         (m_axi.arid),
    .s_axi_araddr       (m_axi.araddr),
    .s_axi_arlen        (m_axi.arlen),
    .s_axi_arsize       (m_axi.arsize),
    .s_axi_arburst      (m_axi.arburst),
    .s_axi_arlock       (m_axi.arlock),
    .s_axi_arcache      (m_axi.arcache),
    .s_axi_arprot       (m_axi.arprot),
    .s_axi_arvalid      (m_axi.arvalid),
    .s_axi_arready      (m_axi.arready),
    .s_axi_rid          (m_axi.rid),
    .s_axi_rdata        (m_axi.rdata),
    .s_axi_rresp        (m_axi.rresp),
    .s_axi_rlast        (m_axi.rlast),
    .s_axi_rvalid       (m_axi.rvalid),
    .s_axi_rready       (m_axi.rready),

    .m_axil_awaddr      (m_axil_125.awaddr),
    .m_axil_awprot      (m_axil_125.awprot),
    .m_axil_awvalid     (m_axil_125.awvalid),
    .m_axil_awready     (m_axil_125.awready),
    .m_axil_wdata       (m_axil_125.wdata),
    .m_axil_wstrb       (m_axil_125.wstrb),
    .m_axil_wvalid      (m_axil_125.wvalid),
    .m_axil_wready      (m_axil_125.wready),
    .m_axil_bresp       (m_axil_125.bresp),
    .m_axil_bvalid      (m_axil_125.bvalid),
    .m_axil_bready      (m_axil_125.bready),
    .m_axil_araddr      (m_axil_125.araddr),
    .m_axil_arprot      (m_axil_125.arprot),
    .m_axil_arvalid     (m_axil_125.arvalid),
    .m_axil_arready     (m_axil_125.arready),
    .m_axil_rdata       (m_axil_125.rdata),
    .m_axil_rresp       (m_axil_125.rresp),
    .m_axil_rvalid      (m_axil_125.rvalid),
    .m_axil_rready      (m_axil_125.rready)
);

axi_pcie_0 u_axi_pcie_0 (
    .pci_exp_txp        (pci_exp_txp),        // output wire [1 : 0] pci_exp_txp
    .pci_exp_txn        (pci_exp_txn),        // output wire [1 : 0] pci_exp_txn
    .pci_exp_rxp        (pci_exp_rxp),        // input wire [1 : 0] pci_exp_rxp
    .pci_exp_rxn        (pci_exp_rxn),        // input wire [1 : 0] pci_exp_rxn

    .REFCLK             (clk_pcie),                        // input wire REFCLK

    .axi_aresetn        (rstn),              // input wire axi_aresetn
    .user_link_up       (link_up),            // output wire user_link_up
    .axi_aclk_out       (clk_axi),            // output wire axi_aclk_out
    .axi_ctl_aclk_out   (),    // output wire axi_ctl_aclk_out
    .mmcm_lock          (),                  // output wire mmcm_lock
    .interrupt_out      (),          // output wire interrupt_out
    .INTX_MSI_Request   ('0),    // input wire INTX_MSI_Request
    .INTX_MSI_Grant     (),        // output wire INTX_MSI_Grant
    .MSI_enable         (),                // output wire MSI_enable
    .MSI_Vector_Num     ('0),        // input wire [4 : 0] MSI_Vector_Num
    .MSI_Vector_Width   (),    // output wire [2 : 0] MSI_Vector_Width


    .m_axi_awready      (m_axi.awready),    // input wire m_axi_awready
    .m_axi_wready       (m_axi.wready),      // input wire m_axi_wready
    .m_axi_bresp        (m_axi.bresp),        // input wire [1 : 0] m_axi_bresp
    .m_axi_bvalid       (m_axi.bvalid),      // input wire m_axi_bvalid
    .m_axi_arready      (m_axi.arready),    // input wire m_axi_arready
    .m_axi_rdata        (m_axi.rdata),        // input wire [63 : 0] m_axi_rdata
    .m_axi_rresp        (m_axi.rresp),        // input wire [1 : 0] m_axi_rresp
    .m_axi_rlast        (m_axi.rlast),        // input wire m_axi_rlast
    .m_axi_rvalid       (m_axi.rvalid),      // input wire m_axi_rvalid
    .m_axi_awaddr       (m_axi.awaddr),      // output wire [63 : 0] m_axi_awaddr
    .m_axi_awlen        (m_axi.awlen),        // output wire [7 : 0] m_axi_awlen
    .m_axi_awsize       (m_axi.awsize),      // output wire [2 : 0] m_axi_awsize
    .m_axi_awburst      (m_axi.awburst),    // output wire [1 : 0] m_axi_awburst
    .m_axi_awprot       (m_axi.awprot),      // output wire [2 : 0] m_axi_awprot
    .m_axi_awvalid      (m_axi.awvalid),    // output wire m_axi_awvalid
    .m_axi_awlock       (m_axi.awlock),      // output wire m_axi_awlock
    .m_axi_awcache      (m_axi.awcache),    // output wire [3 : 0] m_axi_awcache
    .m_axi_wdata        (m_axi.wdata),        // output wire [63 : 0] m_axi_wdata
    .m_axi_wstrb        (m_axi.wstrb),        // output wire [7 : 0] m_axi_wstrb
    .m_axi_wlast        (m_axi.wlast),        // output wire m_axi_wlast
    .m_axi_wvalid       (m_axi.wvalid),      // output wire m_axi_wvalid
    .m_axi_bready       (m_axi.bready),      // output wire m_axi_bready
    .m_axi_araddr       (m_axi.araddr),      // output wire [63 : 0] m_axi_araddr
    .m_axi_arlen        (m_axi.arlen),        // output wire [7 : 0] m_axi_arlen
    .m_axi_arsize       (m_axi.arsize),      // output wire [2 : 0] m_axi_arsize
    .m_axi_arburst      (m_axi.arburst),    // output wire [1 : 0] m_axi_arburst
    .m_axi_arprot       (m_axi.arprot),      // output wire [2 : 0] m_axi_arprot
    .m_axi_arvalid      (m_axi.arvalid),    // output wire m_axi_arvalid
    .m_axi_arlock       (m_axi.arlock),      // output wire m_axi_arlock
    .m_axi_arcache      (m_axi.arcache),    // output wire [3 : 0] m_axi_arcache
    .m_axi_rready       (m_axi.rready),      // output wire m_axi_rready

    .s_axi_awid         (s_axi.awid),                // input wire [3 : 0] s_axi_awid
    .s_axi_awaddr       (s_axi.awaddr),            // input wire [31 : 0] s_axi_awaddr
    .s_axi_awregion     (s_axi.awregion),        // input wire [3 : 0] s_axi_awregion
    .s_axi_awlen        (s_axi.awlen),              // input wire [7 : 0] s_axi_awlen
    .s_axi_awsize       (s_axi.awsize),            // input wire [2 : 0] s_axi_awsize
    .s_axi_awburst      (s_axi.awburst),          // input wire [1 : 0] s_axi_awburst
    .s_axi_awvalid      (s_axi.awvalid),          // input wire s_axi_awvalid
    .s_axi_awready      (s_axi.awready),          // output wire s_axi_awready
    .s_axi_wdata        (s_axi.wdata),              // input wire [63 : 0] s_axi_wdata
    .s_axi_wstrb        (s_axi.wstrb),              // input wire [7 : 0] s_axi_wstrb
    .s_axi_wlast        (s_axi.wlast),              // input wire s_axi_wlast
    .s_axi_wvalid       (s_axi.wvalid),            // input wire s_axi_wvalid
    .s_axi_wready       (s_axi.wready),            // output wire s_axi_wready
    .s_axi_bid          (s_axi.bid),                  // output wire [3 : 0] s_axi_bid
    .s_axi_bresp        (s_axi.bresp),              // output wire [1 : 0] s_axi_bresp
    .s_axi_bvalid       (s_axi.bvalid),            // output wire s_axi_bvalid
    .s_axi_bready       (s_axi.bready),            // input wire s_axi_bready
    .s_axi_arid         (s_axi.arid),                // input wire [3 : 0] s_axi_arid
    .s_axi_araddr       (s_axi.araddr),            // input wire [31 : 0] s_axi_araddr
    .s_axi_arregion     (s_axi.arregion),        // input wire [3 : 0] s_axi_arregion
    .s_axi_arlen        (s_axi.arlen),              // input wire [7 : 0] s_axi_arlen
    .s_axi_arsize       (s_axi.arsize),            // input wire [2 : 0] s_axi_arsize
    .s_axi_arburst      (s_axi.arburst),          // input wire [1 : 0] s_axi_arburst
    .s_axi_arvalid      (s_axi.arvalid),          // input wire s_axi_arvalid
    .s_axi_arready      (s_axi.arready),          // output wire s_axi_arready
    .s_axi_rid          (s_axi.rid),                  // output wire [3 : 0] s_axi_rid
    .s_axi_rdata        (s_axi.rdata),              // output wire [63 : 0] s_axi_rdata
    .s_axi_rresp        (s_axi.rresp),              // output wire [1 : 0] s_axi_rresp
    .s_axi_rlast        (s_axi.rlast),              // output wire s_axi_rlast
    .s_axi_rvalid       (s_axi.rvalid),            // output wire s_axi_rvalid
    .s_axi_rready       (s_axi.rready),            // input wire s_axi_rready

    .s_axi_ctl_awaddr   (s_axi_ctl.awaddr),    // input wire [31 : 0] s_axi_ctl_awaddr
    .s_axi_ctl_awvalid  (s_axi_ctl.awvalid),  // input wire s_axi_ctl_awvalid
    .s_axi_ctl_awready  (s_axi_ctl.awready),  // output wire s_axi_ctl_awready
    .s_axi_ctl_wdata    (s_axi_ctl.wdata),      // input wire [31 : 0] s_axi_ctl_wdata
    .s_axi_ctl_wstrb    (s_axi_ctl.wstrb),      // input wire [3 : 0] s_axi_ctl_wstrb
    .s_axi_ctl_wvalid   (s_axi_ctl.wvalid),    // input wire s_axi_ctl_wvalid
    .s_axi_ctl_wready   (s_axi_ctl.wready),    // output wire s_axi_ctl_wready
    .s_axi_ctl_bresp    (s_axi_ctl.bresp),      // output wire [1 : 0] s_axi_ctl_bresp
    .s_axi_ctl_bvalid   (s_axi_ctl.bvalid),    // output wire s_axi_ctl_bvalid
    .s_axi_ctl_bready   (s_axi_ctl.bready),    // input wire s_axi_ctl_bready
    .s_axi_ctl_araddr   (s_axi_ctl.araddr),    // input wire [31 : 0] s_axi_ctl_araddr
    .s_axi_ctl_arvalid  (s_axi_ctl.arvalid),  // input wire s_axi_ctl_arvalid
    .s_axi_ctl_arready  (s_axi_ctl.arready),  // output wire s_axi_ctl_arready
    .s_axi_ctl_rdata    (s_axi_ctl.rdata),      // output wire [31 : 0] s_axi_ctl_rdata
    .s_axi_ctl_rresp    (s_axi_ctl.rresp),      // output wire [1 : 0] s_axi_ctl_rresp
    .s_axi_ctl_rvalid   (s_axi_ctl.rvalid),    // output wire s_axi_ctl_rvalid
    .s_axi_ctl_rready   (s_axi_ctl.rready)    // input wire s_axi_ctl_rready
);

endmodule